/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.data;

import javax.swing.SwingWorker;

import net.sf.genomeview.gui.dialog.GVProgressBar;
import net.sf.jannot.source.DataSource;

/**
 * A general SwingWorker to use for loading and saving data without blocking the
 * EDT. This class should be extended, where T is the return type of the method
 * that is executed. doInBackground() has to be overridden containing the long
 * running task. A GVWorker is a ProgressListener, so it is given to the
 * DataSource object that passed it setProgress(int, String) messages (which are
 * fired to any PropertyChangeListener listening to this Worker).
 * 
 * @author thpar
 * @author Thomas Abeel
 * 
 */
public abstract class DataSourceWorker extends SwingWorker<Void, Void> {

	protected DataSource source;

	protected Model model;

	protected GVProgressBar pb;

	public DataSourceWorker(DataSource source, Model model) {
		this.source = source;
		this.model = model;
		model.getWorkerManager().start(this);
		this.pb = new GVProgressBar("Background task", "Doing background task", model.getGUIManager().getParent());
	}

	@Override
	protected void done() {
		pb.done();
		model.getWorkerManager().done(this);
	}

	@Override
	protected abstract Void doInBackground();

}
