/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.data;

/**
 * Types of notifications that can be send around between observables and
 * observers. This is useful to let some observers only react to certain
 * notification types.
 * 
 * @author Thomas
 * 
 */
public enum NotificationTypes {
	/**
	 * Notification to indicate that there is a large change in the model which
	 * may affect all observers. Generally observers should react on this type
	 * of notification.
	 */
	GENERAL,
	
	/**
	 * Indicates the event where a user selects another translation table.
	 */
	TRANSLATIONTABLECHANGE,
	
	/**
	 * Indicates that updateTracks was called.
	 */
	UPDATETRACKS,
	
	/**
	 * The entry selection changed
	 */
	ENTRYCHANGED,
	
	/**
	 * Indicates that a feature was edited, added or removed
	 * 
	 */
	JANNOTCHANGE, 
	
	/**
	 * Indicates that there has been a change to a dialog window, either opened or closed.
	 */
	DIALOGCHANGE,
	
	/**
	 * Indicates an exception has occurred somewhere.
	 */
	EXCEPTION
	;

}
