/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.gui.menu.edit;

import java.awt.event.ActionEvent;
import java.util.HashSet;
import java.util.Observable;
import java.util.Observer;
import java.util.Set;
import java.util.SortedSet;

import net.sf.genomeview.data.Model;
import net.sf.genomeview.gui.menu.AbstractModelAction;
import net.sf.jannot.Data;
import net.sf.jannot.Feature;
import net.sf.jannot.Location;
import net.sf.jannot.MemoryFeatureAnnotation;

/**
 * 
 * @author Thomas Abeel
 * 
 */
public class RemoveLocationAction extends AbstractModelAction implements Observer {

	private static final long serialVersionUID = -5857913546086864524L;

	public RemoveLocationAction(Model model) {
		super("Remove selected location", model);
	}

	public void actionPerformed(ActionEvent arg0) {

		Set<Location> toRemove = new HashSet<Location>();
		toRemove.addAll(model.selectionModel().getLocationSelection());
		model.selectionModel().clearLocationSelection();
		for (Location rf : toRemove) {
			Feature f = rf.getParent();
			/* If there are more locations, remove the selected one */
			if (f.location().length > 1)
				f.removeLocation(rf);
			/* If this is the last location of the feature, remove the feature instead*/
			else {
				Data<?>d=model.vlm.getSelectedEntry().get(f.type());
				if(d instanceof MemoryFeatureAnnotation){
					MemoryFeatureAnnotation mf=(MemoryFeatureAnnotation)d;
					mf.remove(f);
				}
				
			}

		}

	}

	public void update(Observable o, Object arg) {
		SortedSet<Location> set = model.selectionModel().getLocationSelection();
		setEnabled(set.size() > 0);

	}

}
