/**
 * This file is part of GenomeView, a genome browser and annotation curator
 * 
 * Copyright (C) 2007-2013 Thomas Abeel and contributors
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 * Project: http://genomeview.org/
 */
package net.sf.genomeview.gui.search;

import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;

import javax.swing.JButton;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextArea;

import net.sf.genomeview.data.Model;
import net.sf.genomeview.gui.components.TypeCombo;
import net.sf.jannot.Feature;
import net.sf.jannot.Location;
import be.abeel.gui.TitledComponent;

/**
 * 
 * @author Thomas Abeel
 * 
 */
class OverlapSearchPane extends SearchPanel {

	private static final long serialVersionUID = 2986333484335063190L;

	public OverlapSearchPane(final Model model) {
		gc.weightx = 1;
		gc.weighty = 0;
		gc.fill = GridBagConstraints.BOTH;
		final JTextArea seq = new JTextArea(7, 30);
		super.setFocusField(seq);
		final TypeCombo sourceType = new TypeCombo(model);
		final TypeCombo targetType = new TypeCombo(model);
		final OverlapSearchResultModel srm = new OverlapSearchResultModel(model);
		final JTable results = new JTable(srm);
		results.addMouseListener(new MouseAdapter() {
			@Override
			public void mouseClicked(MouseEvent e) {
				int row = results.getSelectedRow();
				Feature f = srm.getFeature(row);
				model.selectionModel().setLocationSelection(f);
				if (f.length() < model.vlm.getVisibleLocation().length())
					model.vlm.center((f.end() + f.start()) / 2);
				else {
					double border = 0.05 * (f.end() - f.start());
					model.vlm.setAnnotationLocationVisible(new Location((int) (f.start() - border), (int) (f.end() + border)), true);
				}

			}
		});
		JButton searchButton = new JButton("Search");
		searchButton.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				srm.clear();
				srm.search(sourceType.getTerm(), targetType.getTerm());

			}

		});

		gc.gridwidth = 2;
		add(new TitledComponent("Overlap between type ", sourceType), gc);
		gc.gridy++;
		add(new TitledComponent("and type", targetType), gc);
		gc.gridy++;

		add(searchButton, gc);

		gc.gridwidth = 2;

		gc.gridy++;
		gc.weighty = 1;
		add(new JScrollPane(results), gc);

	}
}